package jdk.management.resource.internal.inst;

import jdk.internal.instrumentation.InstrumentationMethod;
import jdk.management.resource.ResourceRequest;
import jdk.management.resource.ResourceRequestDeniedException;
import java.io.IOException;
import jdk.management.resource.internal.ApproverGroup;
import jdk.management.resource.internal.ResourceIdImpl;
import jdk.internal.instrumentation.TypeMapping;
import jdk.internal.instrumentation.InstrumentationTarget;

@InstrumentationTarget("java.net.SocketOutputStream")
@TypeMapping(from = "jdk.management.resource.internal.inst.SocketOutputStreamRMHooks$AbstractPlainSocketImpl", to = "java.net.AbstractPlainSocketImpl")
public final class SocketOutputStreamRMHooks
{
    private SocketOutputStreamRMHooks.AbstractPlainSocketImpl impl;
    
    public SocketOutputStreamRMHooks() {
        this.impl = null;
    }
    
    @InstrumentationMethod
    private void socketWrite(byte[] array, int n, int n2) throws IOException {
        if (n2 < 0) {
            this.socketWrite(array, n, n2);
            return;
        }
        ResourceIdImpl of = ResourceIdImpl.of(Integer.valueOf(this.impl.localport));
        ResourceRequest approver = ApproverGroup.SOCKET_WRITE_GROUP.getApprover(this);
        long request;
        try {
            request = approver.request((long)n2, of);
            if (request < (long)n2) {
                throw new IOException("Resource limited: insufficient bytes approved");
            }
        }
        catch (ResourceRequestDeniedException cause) {
            throw new IOException("Resource limited", cause);
        }
        int n3 = 0;
        try {
            this.socketWrite(array, n, n2);
            n3 = n2;
        }
        finally {
            approver.request(-(request - (long)n3), of);
        }
        return;
    }
    
    static class AbstractPlainSocketImpl
    {
        protected int localport;
    }
}
